'use strict';

var global = typeof self !== 'undefined' ? self : globalThis;

var _defineProperty = require('@babel/runtime/helpers/defineProperty');
var Hls = require('hls.js');
var common = require('@newstudios/common');
var index = require('./index-1eafae6d.js');
require('delegates');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _defineProperty__default = /*#__PURE__*/_interopDefaultLegacy(_defineProperty);
var Hls__default = /*#__PURE__*/_interopDefaultLegacy(Hls);

const supportMSE = typeof window === 'undefined' ? false : Hls__default["default"].isSupported();
class HlsPlayer extends index.CorePlayer {
  constructor(video, source, fastSwitch = true, capLevelToPlayerSize = false) {
    super(video, source);

    _defineProperty__default["default"](this, "_nextLevel", -1);

    _defineProperty__default["default"](this, "_readyDisposable", this._register(new common.MutableDisposable()));

    this._fastSwitchEnabled = fastSwitch;

    if (supportMSE) {
      const hlsPlayer = new Hls__default["default"]({
        autoStartLoad: false,
        capLevelToPlayerSize
      });
      this._hlsPlayer = hlsPlayer;

      this._register(common.toDisposable(() => hlsPlayer.destroy()));
    }
  }

  get levels() {
    var _this$_hlsPlayer;

    return ((_this$_hlsPlayer = this._hlsPlayer) === null || _this$_hlsPlayer === void 0 ? void 0 : _this$_hlsPlayer.levels) || [];
  }

  get currentLevel() {
    var _this$_hlsPlayer2, _this$_hlsPlayer3;

    return (_this$_hlsPlayer2 = this._hlsPlayer) === null || _this$_hlsPlayer2 === void 0 ? void 0 : _this$_hlsPlayer2.levels[(_this$_hlsPlayer3 = this._hlsPlayer) === null || _this$_hlsPlayer3 === void 0 ? void 0 : _this$_hlsPlayer3.currentLevel];
  }

  get nextLevel() {
    var _this$_hlsPlayer4;

    return (_this$_hlsPlayer4 = this._hlsPlayer) === null || _this$_hlsPlayer4 === void 0 ? void 0 : _this$_hlsPlayer4.levels[this._nextLevel];
  }

  get autoQualityEnabled() {
    const hlsPlayer = this._hlsPlayer;

    if (hlsPlayer) {
      return hlsPlayer.autoLevelEnabled;
    }

    return true;
  }

  levelToQuality(hlsLevel) {
    const level = {
      bitrate: hlsLevel.bitrate,
      width: hlsLevel.width,
      height: hlsLevel.height
    };

    if (hlsLevel.videoCodec) {
      level.type = 'video';
    }

    return level;
  }

  findLevelIndexByQualityLevel(playLevel) {
    // bitrate match
    const idx = this.levels.findIndex(level => level.bitrate === playLevel.bitrate);

    if (idx >= 0) {
      return idx;
    } // short side match


    const shortSide = Math.min(playLevel.width, playLevel.height);
    return this.levels.findIndex(level => Math.min(level.width, level.height) === shortSide);
  }

  setAutoQualityState(auto) {
    const hlsPlayer = this._hlsPlayer;

    if (!hlsPlayer) {
      return;
    }

    if (auto) {
      this._nextLevel = -1;

      if (this._fastSwitchEnabled) {
        hlsPlayer.nextLevel = -1;
      } else {
        hlsPlayer.loadLevel = -1;
      }
    } else {
      this._nextLevel = hlsPlayer.currentLevel;
    }

    this._readyDisposable.value = undefined;
  }

  setNextLevelIndex(index) {
    const hlsPlayer = this._hlsPlayer;

    if (!hlsPlayer) {
      return;
    }

    this._nextLevel = index;

    if (this._fastSwitchEnabled) {
      hlsPlayer.nextLevel = index;
    } else {
      hlsPlayer.loadLevel = index;
    }

    this._readyDisposable.value = undefined;
  }

  onInit(video, source) {
    const hlsPlayer = this._hlsPlayer;

    if (hlsPlayer) {
      const disposables = new common.DisposableStore();
      const onManifestParsed = common.Event.fromNodeEventEmitter(hlsPlayer, Hls__default["default"].Events.MANIFEST_PARSED);
      const onLevelsUpdated = common.Event.fromNodeEventEmitter(hlsPlayer, Hls__default["default"].Events.LEVEL_UPDATED);
      const onLevelSwitched = common.Event.fromNodeEventEmitter(hlsPlayer, Hls__default["default"].Events.LEVEL_SWITCHED);
      const onLevelSwitching = common.Event.fromNodeEventEmitter(hlsPlayer, Hls__default["default"].Events.LEVEL_SWITCHING);
      onLevelsUpdated(this.updatePlayList, this, disposables);
      onManifestParsed(this.updatePlayList, this, disposables);
      onManifestParsed(this.setReady, this, disposables);
      onManifestParsed(() => {
        hlsPlayer.startLoad();
        video.autoplay && video.play();
      });
      onLevelSwitching(this.updateNextQualityLevel, this, disposables);
      onLevelSwitched(this.updateQualityLevel, this, disposables);
      hlsPlayer.loadSource(source.src);
      hlsPlayer.attachMedia(video);

      this._register(disposables);
    } else {
      this.video.src = source.src;

      if (!this.video.canPlayType(source.mime)) {
        common.onUnexpectedError(new Error('hlsplayer src not supported: ' + source.src + ', mime: ' + source.mime));
      }

      this.updatePlayList();
    }
  }

  setInitialBitrate(bitrate) {
    const hlsPlayer = this._hlsPlayer;

    if (!hlsPlayer) {
      return;
    }

    if (this.ready) {
      this._readyDisposable.value = undefined;
      let index = 0;
      const levels = this.levels.slice().sort((a, b) => a.bitrate - b.bitrate);

      for (let i = 0; i < levels.length; i++) {
        if (levels[i].bitrate < bitrate) {
          index = i;
        } else {
          break;
        }
      }

      const startLevel = levels[index];
      hlsPlayer.startLevel = this.levels.findIndex(l => l.bitrate === startLevel.bitrate);
    } else {
      this._readyDisposable.value = this.onReady(() => this.setInitialBitrate(bitrate));
    }
  }

  setCapLevelToPlayerSize(capLevelToPlayerSize) {
    if (this._hlsPlayer) {
      this._hlsPlayer.capLevelToPlayerSize = capLevelToPlayerSize;
    }
  }

  get capLevelToPlayerSize() {
    var _this$_hlsPlayer$capL, _this$_hlsPlayer5;

    return (_this$_hlsPlayer$capL = (_this$_hlsPlayer5 = this._hlsPlayer) === null || _this$_hlsPlayer5 === void 0 ? void 0 : _this$_hlsPlayer5.capLevelToPlayerSize) !== null && _this$_hlsPlayer$capL !== void 0 ? _this$_hlsPlayer$capL : false;
  }

  get bandwidthEstimate() {
    const player = this._hlsPlayer;
    return (player === null || player === void 0 ? void 0 : player.bandwidthEstimate) || NaN;
  }

  get name() {
    return `HLSPlayer (${Hls__default["default"].version})`;
  }

  get supportAutoQuality() {
    return true;
  }

}

exports.HlsPlayer = HlsPlayer;
